# window.py
#
# Copyright 2024 Hari Rana (TheEvilSkeleton)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later


import logging

from typing import Any, Optional, cast

from gi.repository import Adw, Gtk, Gio, GObject, GLib

from refine.widgets.adjustment import RefineAdjustment
from refine.widgets.panel_row import RefinePanelRow
from refine.widgets.data_combo_row import RefineDataComboRow
from refine.widgets.flags_switch_row import RefineFlagsSwitchRow
from refine.widgets.switch_row import RefineSwitchRow
from refine.widgets.font_row import RefineFontRow
from refine.widgets.reset_button import RefineResetButton
from refine.widgets.value_switch_row import RefineValueSwitchRow


@Gtk.Template(resource_path="/page/tesk/Refine/widgets/panel-page.ui")
class RefinePanelPage(Adw.PreferencesPage):
    """Panel page containing options."""

    __gtype_name__ = "RefinePanelPage"


@Gtk.Template(resource_path="/page/tesk/Refine/window.ui")
class RefineWindow(Adw.ApplicationWindow):
    """The main window singleton class."""

    __gtype_name__ = "RefineWindow"

    panels_list_box = Gtk.Template.Child()  # type: ignore [misc]
    main_navigation_split_view = Gtk.Template.Child()  # type: ignore [misc]

    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

        self.needs_restart: list = []

        panels_schema = [
            "/page/tesk/Refine/sections/appearance.ui",
            "/page/tesk/Refine/sections/mouse-touchpad.ui",
            "/page/tesk/Refine/sections/shell-compositor.ui",
            "/page/tesk/Refine/sections/sound.ui",
        ]

        panels_list = Gtk.StringList.new(panels_schema)

        self.panels_list_box.bind_model(panels_list, self.__create_row_func)
        self.panels_list_box.connect("row-activated", self.__set_preferences_page_cb)

        first_row = self.panels_list_box.get_first_child()
        self.__set_preferences_page(first_row)

    def __create_row_func(self, panel: Gtk.StringObject, *args) -> Optional[Gtk.Widget]:
        builder = Gtk.Builder.new_from_resource(panel.get_string())
        row = RefinePanelRow()
        if not (page := builder.get_object("page")):
            return None

        row.page = page  # type: ignore [method-assign]

        row.page.bind_property(
            "icon-name", row, "icon-name", GObject.BindingFlags.SYNC_CREATE
        )
        row.page.bind_property("title", row, "title", GObject.BindingFlags.SYNC_CREATE)

        return row

    def __set_preferences_page(self, row: RefinePanelRow, *args) -> None:
        navigation_page = self.main_navigation_split_view.get_content()
        toolbar_view = navigation_page.get_child()

        toolbar_view.set_content(row.page)
        navigation_page.set_title(row.page.get_title())

    def __set_preferences_page_cb(self, _: Any, row: RefinePanelRow, *args) -> None:
        self.__set_preferences_page(row)
        self.main_navigation_split_view.set_show_content(True)

    # TODO: Find a way to request a logout dialog
    @GObject.Signal
    def restart_items_changed(self, widget: Gtk.Widget) -> None:
        logging.info(f"{self.__class__.__name__}::restart-items-changed signal emitted")

        if not (panel := cast(RefinePanelPage, widget.get_ancestor(RefinePanelPage))):
            return

        if not (banner := panel.get_banner()):  # type: ignore [attr-defined]
            return

        try:
            self.needs_restart.remove(widget)
            banner.set_revealed(False)
        except ValueError:
            self.needs_restart.append(widget)
            banner.set_revealed(True)
